function C = threeDmulti_rot(A,B,type)

% This function multiplies 3 dimensional rotation matrices in form A(3*3*n)
% with position vectors in form B(n*3). This is mainly used in coordinate
% transformation between intertial and earth-fixed frames.
% 
% Input:    R = rotation matrices           [3x3xN]
%           B = vectors to be rotated       [Nx3]
%        type = 'forward' or 'backward' ('backward' for inverse rotation);
%                    default is 'forward'
% 
% Output:   C = rotated vectors             [Nx3]
% 
% Example:      R = RotmatICRF2SF(SCA1B_A(:,2:5));
%               v = acc_ICRF;
%    -->   acc_SF = threeDmulti_rot(R,v);
% 
% Written by Majid Naeimi, 7 April 2015, IfE, Hanover, Germany
% Last updated by Henry Wegener, AEI Hannover, 2016-07-08.


if ~( size(A,1) == 3 && size(A,2) == 3 && size(B,2) == 3 )
    error('Dimensions of input arguments are incorrect. Rotation matrices (first argument) must be of size 3x3xN \n Input vectors (second argument) must be of size Nx3 !')
elseif ( size(A,3) > 1 && size(B,1) == 1 )
    B = repmat(B,size(A,3),1);
elseif ~( size(A,3) == size(B,1) )
    error('Length of vector (second argument) must be the same as for the matrices OR it must be 1 !');
end

if ( nargin == 2 )
    type = 'forward';
end

Ap = reshape(A,9,[]);
switch type
    case 'forward'
        C = [Ap(1,:)'.*B(:,1)+Ap(4,:)'.*B(:,2)+Ap(7,:)'.*B(:,3) Ap(2,:)'.*B(:,1)+Ap(5,:)'.*B(:,2)+Ap(8,:)'.*B(:,3) Ap(3,:)'.*B(:,1)+Ap(6,:)'.*B(:,2)+Ap(9,:)'.*B(:,3)];
    case 'backward'
        C = [Ap(1,:)'.*B(:,1)+Ap(2,:)'.*B(:,2)+Ap(3,:)'.*B(:,3) Ap(4,:)'.*B(:,1)+Ap(5,:)'.*B(:,2)+Ap(6,:)'.*B(:,3) Ap(7,:)'.*B(:,1)+Ap(8,:)'.*B(:,2)+Ap(9,:)'.*B(:,3)];
    otherwise
        error(['Unknown type: ',type,'. type must be either "forward" (for forward rotation) \n or "backward" (for inverse rotation)!'])
end


end