function [dX] = deriv(time0,X0, deriv_params)

% DERIV reads state and STM as one vector and calculates the
% derivative of this vector to be integrated input and output are in
% inertial frame.
%
% abbreviation: spherical harmonic coefficients(SHC)
% state transition matrix(STM):PHI
% acceleration:acc
%
% Input:        time0 = GRACE time
%               X0 = state vector and PHI for time0 including [RA; RdotA; RB; RdotB; PHI]
%               [GRACE A position, GRACE A velocity, GRACE B position, GRACE B velocity, elements of PHI]
%               deriv_params = struct of all additional parameters to the
%               deriv function:
%                   field0 = SHC of background gravity field
%                   data_plm0 = initialized Legendre polynomials
%                   GM = GM of background gravity field
%                   ae = Earth radius of background gravity field
%                   lmax_grav = D/O of SHC of background gravity field
%                   lmax_vari = D/O of SHC to be estimated
%                   noc0 = total number of SHC to be estimated
%
% Output:       dX = derivatives of X0 [RdotA; RddotA; RdotB; RddotB; PHIdot]
%               [GRACE A velocity, GRACE A acc, GRACE B velocity, GRACE B acc, elements of PHIdot]
%
% Main functions
% - grtenpshs: calculates acc and acc partials w.r.t position:Rddot,Rddot/R
% - llpartialgradV :calculates acc partials w.r.t SHC:Rddot/Klm
% Other functions
% - Ri2e: Transformation from inertial to Earth fixed frame
% - cart2sph:cartesian to spherical coordinates
%
% Written by Neda Darbeheshti, AEI Hannover, 2018-06
% Major bug corrected for tensV1 and tensV2 by Florian, ZARM Uni Bremen,2018-07
% Optimized and brushed up by Florian Wöske, ZARM Uni Bremen,2018-08-02

%%

field = deriv_params.field;
data_plm = deriv_params.data_plm;
GM = deriv_params.GM;
ae = deriv_params.ae;
lmax_grav = deriv_params.lmaxf;
lmax_vari = deriv_params.lmaxcs;
noc = deriv_params.noc;

% global AMAT phi_square phidot0
% rotation matrices from inertial to Earth-fixed frame
RotMat=Ri2e(time0);

% rotation matrices reformatting
RotMate2i=reshape(RotMat,1,9);

% allocate dX
dX = zeros(length(X0),1);

%% Acceleration and Velocity: EOM for GRACE A and B

% GRACE A
% transform position to the Earth-fixed frame
% pos1 = threeDmulti_rot(RotMat,X0(1:3)','forward');
pos1 = RotMat*X0(1:3);

% transform Cartesian to spherical coordinates
[lam1,phi1,r1] = cart2sph(pos1(1),pos1(2),pos1(3));

% calculate acceleration along the orbit: gradV1 and acc partials w.r.t. position: tensV1
[gradV1,tensV1] = grtenpshs(field,lam1,phi1,r1,data_plm,GM,ae,lmax_grav);

% transform acc partials w.r.t. position to inertial frame
tensV1 = RotMat'*tensV1*RotMat;

% Extract velocity from state vector
rdot_1 = X0(4:6); %m/sec, GRACE A velocity

% transform acc to inertial frame
rddot_1 = RotMat'*gradV1; %m/s2, GRACE A acc

% GRACE B
% transform position to the Earth-fixed frame
pos2 = RotMat*X0(7:9);

% transform Cartesian to spherical coordinates
[lam2,phi2,r2] = cart2sph(pos2(1),pos2(2),pos2(3));

% calculate acceleration along the orbit: gradV2 and acc partials w.r.t. position: tensV2
[gradV2,tensV2] = grtenpshs(field,lam2,phi2,r2,data_plm,GM,ae,lmax_grav);

% transform acc partials w.r.t. position to inertial frame
tensV2 = RotMat'*tensV2*RotMat;

% Extract velocity from state vector
rdot_2 = X0(10:12); %m/sec, GRACE B velocity

% transform acc to inertial frame
rddot_2 = RotMat'*gradV2; %m/s2, GRACE B acc

% filling dX with 12x1 EOM vector
dX(1:12) = [rdot_1; rddot_1; rdot_2; rddot_2];

%% Calculates acc partials w.r.t SHC: Rddot/Klm for each satellite
% It is accumulated like this:
% [c00,c10,c20,c30,c11,c21,c31,c22,c32,c33] (degree_wise).
% This is different from Hu's Fortran function that accumulates the partials 
% like this:
% [c00,c10,c11,c20,c21,c22,c30,c31,c32,c33] (order_wise).

nC = ((lmax_vari+1)^2 + lmax_vari+1)/2;
nS = nC - lmax_vari -1;

dDV_dClm_A_x=zeros(nC,1);
dDV_dSlm_A_x=zeros(nS,1);
dDV_dClm_A_y=zeros(nC,1);
dDV_dSlm_A_y=zeros(nS,1);
dDV_dClm_A_z=zeros(nC,1);
dDV_dSlm_A_z=zeros(nS,1);
dDV_dClm_B_x=zeros(nC,1);
dDV_dSlm_B_x=zeros(nS,1);
dDV_dClm_B_y=zeros(nC,1);
dDV_dSlm_B_y=zeros(nS,1);
dDV_dClm_B_z=zeros(nC,1);
dDV_dSlm_B_z=zeros(nS,1);

% index for right ordering of dC and dS in preallocated dDV_dClm vectors
id_endC = 1;
id_endS = 1;

for m=0:lmax_vari
    [dDV_dClm_A,dDV_dSlm_A] = llpartialgradV(lam1,phi1,r1,lmax_vari,m,[ GM ae],RotMate2i,1);
    [dDV_dClm_B,dDV_dSlm_B] = llpartialgradV(lam2,phi2,r2,lmax_vari,m,[ GM ae],RotMate2i,1);

    id_endC_n = id_endC+lmax_vari-m;

    dDV_dClm_A_x(id_endC:id_endC_n) = squeeze(dDV_dClm_A(:,1,:));
    dDV_dClm_A_y(id_endC:id_endC_n) = squeeze(dDV_dClm_A(:,2,:));
    dDV_dClm_A_z(id_endC:id_endC_n) = squeeze(dDV_dClm_A(:,3,:));

    dDV_dClm_B_x(id_endC:id_endC_n) = squeeze(dDV_dClm_B(:,1,:));
    dDV_dClm_B_y(id_endC:id_endC_n) = squeeze(dDV_dClm_B(:,2,:));
    dDV_dClm_B_z(id_endC:id_endC_n) = squeeze(dDV_dClm_B(:,3,:));

    if m~0;
        id_endS_n = id_endS+lmax_vari-m;

        dDV_dSlm_A_x(id_endS:id_endS_n) = squeeze(dDV_dSlm_A(:,1,:));
        dDV_dSlm_A_y(id_endS:id_endS_n) = squeeze(dDV_dSlm_A(:,2,:));
        dDV_dSlm_A_z(id_endS:id_endS_n) = squeeze(dDV_dSlm_A(:,3,:));

        dDV_dSlm_B_x(id_endS:id_endS_n) = squeeze(dDV_dSlm_B(:,1,:));
        dDV_dSlm_B_y(id_endS:id_endS_n) = squeeze(dDV_dSlm_B(:,2,:));
        dDV_dSlm_B_z(id_endS:id_endS_n) = squeeze(dDV_dSlm_B(:,3,:));
        id_endS = id_endS_n + 1;
    end

    id_endC = id_endC_n + 1;
end

% eliminate C00,C10,C11,S11 in x,y,z directions
dDV_dClm_A_x(1)=[];dDV_dClm_A_x(1)=[];dDV_dClm_A_x(lmax_vari)=[];
dDV_dClm_A_y(1)=[];dDV_dClm_A_y(1)=[];dDV_dClm_A_y(lmax_vari)=[];
dDV_dClm_A_z(1)=[];dDV_dClm_A_z(1)=[];dDV_dClm_A_z(lmax_vari)=[];
dDV_dSlm_A_x(1)=[];dDV_dSlm_A_y(1)=[];dDV_dSlm_A_z(1)=[];

dDV_dClm_B_x(1)=[];dDV_dClm_B_x(1)=[];dDV_dClm_B_x(lmax_vari)=[];
dDV_dClm_B_y(1)=[];dDV_dClm_B_y(1)=[];dDV_dClm_B_y(lmax_vari)=[];
dDV_dClm_B_z(1)=[];dDV_dClm_B_z(1)=[];dDV_dClm_B_z(lmax_vari)=[];
dDV_dSlm_B_x(1)=[];dDV_dSlm_B_y(1)=[];dDV_dSlm_B_z(1)=[];

dDVdklmA=[dDV_dClm_A_x' dDV_dSlm_A_x' ; dDV_dClm_A_y' dDV_dSlm_A_y'; dDV_dClm_A_z' dDV_dSlm_A_z' ];
dDVdklmB=[dDV_dClm_B_x' dDV_dSlm_B_x' ; dDV_dClm_B_y' dDV_dSlm_B_y'; dDV_dClm_B_z' dDV_dSlm_B_z' ];

%% Building A matrix
I = eye(3,3);
Z3 = zeros(3,3);
Zn = zeros(3,noc);

%filling A matrix with partials w.r.t position
AMATs = [Z3 I Z3 Z3; tensV1 Z3 Z3 Z3; Z3 Z3 Z3 I; Z3 Z3 tensV2 Z3];

%filling A matrix with partials w.r.t SHC
AMATc = [Zn; dDVdklmA; Zn ; dDVdklmB];

%size of AMAT is 12*noc
AMAT = [AMATs AMATc];

%% Extract the current PHI:
phi0= reshape(X0(12+1:end), 12, 12+noc);

% phi_square is PHI square matrix
phi_square=eye(12+noc);
[a,b] = size(phi0);
phi_square(1:a,1:b) = phi0;

% PHIdot:
phidot0 = AMAT*phi_square;

% filling rest of dX with PHIdot
dX(12+1:end) = reshape(phidot0,numel(phidot0),1);
end


