%% Gradient and tensor of gravity field
function [gradV,tensV] = grtenpshs(field,lam,phi,r,data,GM,ae,lmax)
% GRTENPSHS calculates the gradient and the tensor of the gravity field
% for two points. It is especially designed to speed up the calculation in
% the orbit integration. The output is the gradient in the Earth-fixed
% frame. 
%
% Input:        field = SHC of gravity field
%               lam,phi,r = sperical coordiantes in Earth fixed frame
%               data = initialized Legendre polynomials
%               GM = GM of gravity field
%               ae = Earth radius of gravity field
%               lmax = D/O of SHC gravity field
% 
% Output:       gradV = Acceleration in Earth fixed frame
%               tensV = Acceleration partials w.r.t. position
%
% Written by Matthias Weigelt


%% CALCULATION
% ---------------------------------------------
GFr1  = GM./(r.^2);
GFrr1 = GFr1./r;
% ---------------------------------------------
m     = 0:lmax;
l     = m'; 
updw1 = (ae./r).^l;                   % Upward continuation term
l     =     l*ones(1,2*lmax+1); l     = l(:);
updw1 = updw1*ones(1,2*lmax+1); updw1 = updw1(:);
% ---------------------------------------------
ml     = lam*m;                           % Sine and cosine arguments
sml    = sin(ml);
cml    = cos(ml);
scml1  = ones(lmax+1,1)*[ fliplr(                   sml(1,2:end))          cml(1,:) ];  scml1  = scml1(:);
csml1  = ones(lmax+1,1)*[ fliplr( m(:,2:end)     .* cml(1,2:end)),    -m .*sml(1,:) ];  csml1  = csml1(:);
cs2ml1 = ones(lmax+1,1)*[ fliplr((m(:,2:end).^2) .* sml(1,2:end)), (m.^2).*cml(1,:) ];  cs2ml1 = cs2ml1(:);
% ---------------------------------------------
tp = tan(phi);
cp = cos(phi);                        % Rotation matrices into the Earth fixed frame
sp = sin(phi);
cl = cml(:,2);
sl = sml(:,2);
V1 = [ cp.*cl  -sp.*cl  -sl; ...
       cp.*sl  -sp.*sl   cl; ...
       sp          cp     0     ];
% ---------------------------------------------
[p1,dp1,ddp1] = plmsp(lmax,pi/2-phi,data);  % Legendre polynomials
p1  = p1(:);
dp1 = dp1(:);
% ---------------------------------------------
gradV1(1,1) = -GFr1        .* field*( updw1.*scml1.* p1.*(l+1) );
gradV1(2,1) =  GFr1        .* field*( updw1.*scml1.*dp1        );
gradV1(3,1) =  GFr1 ./cp(1).* field*( updw1.*csml1.* p1        );
tensV1(1,1) =  GFrr1 .* field*( updw1.*scml1.*p1.*(l+1).*(l+2) );
tensV1(1,2) = -GFrr1 .* field*( updw1.*scml1.*dp1.*(l+1) ) - gradV1(2,1)./r(1);
tensV1(1,3) = -GFrr1./cp(1) .* field*( updw1.*csml1.*p1.*(l+1) ) - gradV1(3,1)./r(1);
tensV1(2,1) = tensV1(1,2);
tensV1(2,2) =  GFrr1 .* field*( updw1.*scml1.*ddp1(:) ) + gradV1(1,1)./r(1);
tensV1(2,3) =  GFrr1./cp(1) .* field*( updw1.*csml1.*dp1 ) + gradV1(3,1)./r(1).*tp(1);
tensV1(3,1) = tensV1(1,3);
tensV1(3,2) = tensV1(2,3);
tensV1(3,3) = -GFrr1./(cp(1).^2) .* field*( updw1.*cs2ml1.*p1 ) + gradV1(1,1)./r(1) - gradV1(2,1)./r(1).*tp(1);
% ---------------------------------------------
gradV = V1*gradV1;
tensV = V1*tensV1*V1';  
end
