function [Hi_tilda,GiR]= hmat(X0,NOCs)

% HMAT makes the Htilde matrix.
% References:-McCullough CM (2017) Gravity Field Estimation for Next Generation Satellite
%             Missions,  page 97, equation (4.4)
%             Liu(2008), page 28, equation (2.51) 
%            -Statistical Orbit Determination (Tapley et al., 2004) 
% 
% Input:
%
%   X0 = state vector including [RA; RdotA; RB; RdotB; PHI]
%   NOCs = total number of coefficients to be estimated
% 
% Outputs:
%
%   Hi_tilda = [dRhodot/dRA; dRhodot/dRdotA; dRhodot/dRB; dRhodot/dRdotB; dRhodot/dKlm]
%   GiR = [Rho Rhodot]
% 
% Main functions
% - inter_sat_dist: calculates range and range rate from position and velocity of two satellites
% 
% Written by Neda Darbeheshti, AEI Hannover, 2018-05-25

%%
% calculate range and range rate 
GiR = inter_sat_dist(X0(1:12));

% calculate line of sight vector
e_12=[(X0(7)-X0(1))/GiR(1) (X0(8)-X0(2))/GiR(1) (X0(9)-X0(3))/GiR(1)];

%% calculate range rate partials w.r.t position
dpdotdx = ((X0(10)-X0(4))/GiR(1)) - ((GiR(2)*(X0(7)- X0(1)))/(GiR(1)^2));
dpdotdy = ((X0(11)-X0(5))/GiR(1)) - ((GiR(2)*(X0(8)- X0(2)))/(GiR(1)^2));
dpdotdz = ((X0(12)-X0(6))/GiR(1)) - ((GiR(2)*(X0(9)- X0(3)))/(GiR(1)^2));

%% calculate range rate partials w.r.t SHC
% Extract the current state transition matrix:
phi = reshape(X0(12+1:end), 12, 12+NOCs);

% Extract the partials from the current state transition matrix:
dr1_dklm=phi(1:3,13:end);
ddotr1_dklm=phi(4:6,13:end);
dr2_dklm=phi(7:9,13:end);
ddotr2_dklm=phi(10:12,13:end);

% form partials according to equation (4.4) of McCullough(2017):
term1=[((X0(10)-X0(4))/GiR(1)) ((X0(11)-X0(5))/GiR(1)) ((X0(12)-X0(6))/GiR(1))];
term2=(dr2_dklm-dr1_dklm)+e_12'*e_12*(dr1_dklm-dr2_dklm);
term3=e_12*(ddotr2_dklm-ddotr1_dklm);
Hi_tilda_klm=term1*term2+term3;

% fill Hi_tilda matrix
Hi_tilda=[-dpdotdx -dpdotdy -dpdotdz -e_12 dpdotdx dpdotdy dpdotdz e_12 Hi_tilda_klm];

