%% Initialize procedure for PLM
function data = initplm(lmax,state)

% INITPLM initialzes a plm calculation. It delivers all the components
% which are repeatidly used and are independent of the variable position
% and velocity.
%
% Usage:
%
%   [usem,fac,didx,root1,root2,droot1,droot2] = initplm(lmax,state)			
%
% Input:
%
%   lmax  - degree (scalar). Integer, representing the maximum degree
%   state - necessary information about the requested quantity
%       0 = only Legendre
%       1 = Legendre + 1st deriv
%       2 = Legendre + 1st deriv + 2nd deriv
%
% Output:
%
%   usem - used maximum degree
%   fac - factor for sectorial recursion
%   didx - index to the diagonal
%   root1 - first  root for Legendre polynomial
%   root2 - second root for Legendre polynomial
%   droot1 - first  root for first derivative of Legendre polynomial
%   droot2 - second root for first derivative of Legendre polynomial

% Preliminaries
if state == 0, 
    data.usem = lmax;
elseif state == 1,
    data.usem = lmax+1;
elseif state == 2,
    data.usem = lmax+2;
end

% factor for sectorial recursion
mm  = 2*(1:data.usem);
data.fac = [1 sqrt(2*cumprod((mm+1)./mm))];

% Index to the diagonal elements
didx  = 1:data.usem+1;
data.didx  = sub2ind([data.usem+1 data.usem+1],didx,didx);

% Roots for Legendre polynomial
[m,l] = meshgrid(0:data.usem,0:data.usem);
m     = tril(m);
l     = tril(l);
denom = (l-m).*(l+m);
denom(denom < 0) = 1;
data.root1 = sqrt( tril((2*l+1).*(2*l-1)          ./ denom             ,-1 ));
data.root1 = [data.root1(2:end,:); data.root1(1,:)];
data.root2 = sqrt( tril((2*l+1).*(l+m-1).*(l-m-1) ./ ((2*l-3) .* denom),-1 ));
data.root2 = [data.root2(3:end,:); data.root2(1:2,:)];

% Roots for the first derivative
if state >= 1
    data.droot1 = zeros(data.usem+1);
    data.droot2 = zeros(data.usem+1);
    data.droot1(:,1) = -sqrt((l(:,1)+1).*l(:,1)./2);
    if lmax > 0
        data.droot1(:,2) = sqrt((l(:,2)+1).*l(:,2).*2 );
        data.droot2(2:end,2) = sqrt((l(2:end,2)-1).*(l(2:end,2)+2));
        if lmax > 1
            ll = l(3:end,3:end);
            mm = m(3:end,3:end);
            data.droot1(3:end,3:end) = sqrt((ll+mm).*(ll-mm+1));
            data.droot2(3:end,3:end) = sqrt((ll-mm).*(ll+mm+1));
        end
    end
end
end
