function [dDV_dClm,dDV_dSlm] = llpartialgradV(lam,phi,r,lmax,m,const,drtype,maxlength)

% LLPARTIALGRADV determines the partial derivative of the gradient of the
% gravity field w.r.t. the coefficients. The output is either in the
% Earth-fixed frame of may be rotated to any other frame. The output matrix
% is three-dimensional since the three coordinate directions exist for
% every time-point and every degree l. The order m is fixed. Thus the
% calculation is pointwise and designed for e.g. calculations along the
% orbit.
%
% Call: [dDV_dClm,dDV_dSlm] = llpartialgradV(lam,phi,r,lmax,m,const,drtype,maxlgth)
%
% Parameter:
%  lam     [t,1]   longitude in [rad] 
%  phi     [t,1]   latitude in [rad]
%  r       [t,1]   radius in [m]
%  lmax    [1,1]   maximum degree to consider
%  const   [1,2]   constants GM (element 1) and ae (element 2)
%  drtype  [str]   defines the type of output
%                  - 'xyz'   are the derivatives towards {x,y,z}
%                            but in the cartesian frame (default) 
%                  - 'lscs'  are the derivatives towards {x,y,z}
%                            but in the local spherical coordinate frame
%                  - R [t,9] rotation matrix from 'xyz' to desired frame 
%  maxlgth [1,1]   maximum length of arcs during processing
%
% Output:   dDV_dClm = degree wise Acceleration partials w.r.t. C
%           dDV_dSlm = degree wise Acceleration partials w.r.t. S
%
% Written by Matthias Weigelt, 2008-05-08

%% Input check
narginchk(5,8)
if nargin < 8 || isempty(maxlength), maxlength = 512;    end
if nargin < 7 || isempty(drtype),    drtype    = 'lscs'; end
if nargin < 6 || isempty(const),     const     = [];     end
% -------------------------------------
% load necessary constants
if isempty(const)
    constants;
elseif numel(const) == 2
    GM = const(1);
    ae = const(2);
else
    error('CONST must be a 2x1 vector or empty.')
end

%% PREPARATION
% -------------------------------------
% prepare the coordinates
lam = lam(:);     
phi = phi(:);
r   = r(:);
if numel(phi) ~= numel(lam) || numel(r) ~= numel(lam), 
    error('Coordinates ''r'', ''phi'' and ''lam'' must have the same length.'); 
end
% -------------------------------------
% prepare the output
dDV_dClm = NaN(length(lam),3,numel(m:lmax));
dDV_dSlm = NaN(length(lam),3,numel(m:lmax));

%% CALCULATION
didx = 1:numel(lam);
didx = didx(all(~isnan([lam phi r]),2));
% -------------------------------------
% process data (piecewise)
parts = ceil(numel(lam)/maxlength);
TK    = GM./(2.*ae.^2);
for ridx = 1:parts
    % -------------------------------------
    % select the data and get the co-latitude 
    idx = didx((ridx-1)*maxlength+1:min(ridx*maxlength,numel(didx)));
    th  = (pi/2 - phi(idx)).*180./pi;                   % unit is degree
    % -------------------------------------
    l   = m:lmax;
    TF  = bsxfun(@power,ae./r(idx) * ones(1,numel(l)),l+2);  % matrix of size(length(lam),length(l))
    TSQ = realsqrt((2.*l+1)./(2.*l+3));
    if m == 0
        pFlm  = TSQ.*realsqrt((l+m+1).*(l+m+2).*2);
        pPlm  = TF.*(plm(l+1,m+1,th));
        pcos  = cos((m+1).*lam(idx));
        psin  = sin((m+1).*lam(idx));
        % ----------------------------
        oFlm  = TSQ.*realsqrt((l-m+1).*(l+m+1));
        oPlm  = TF.*(plm(l+1,m,th));
        ocos  = cos(m.*lam(idx));
        % ----------------------------
        dDV_dClm(idx,1,:) = bsxfun(@times,   -TK.*pFlm,bsxfun(@times,pPlm,pcos));
        dDV_dClm(idx,2,:) = bsxfun(@times,   -TK.*pFlm,bsxfun(@times,pPlm,psin));
        dDV_dClm(idx,3,:) = bsxfun(@times,-2.*TK.*oFlm,bsxfun(@times,oPlm,ocos));
        % ----------------------------
        dDV_dSlm(idx,1,:) = bsxfun(@times,   -TK.*pFlm,bsxfun(@times,pPlm,psin));
        dDV_dSlm(idx,2,:) = bsxfun(@times,    TK.*pFlm,bsxfun(@times,pPlm,pcos));
        dDV_dSlm(idx,3,:) = zeros(size(oPlm));

    elseif m == 1
        pFlm  = TSQ.*realsqrt((l+m+1).*(l+m+2));
        pPlm  = TF.*(plm(l+1,m+1,th));
        pcos  = cos((m+1).*lam(idx));
        psin  = sin((m+1).*lam(idx));
        % ----------------------------
        oFlm  = TSQ.*realsqrt((l-m+1).*(l+m+1));
        oPlm  = TF.*(plm(l+1,m,th));
        ocos  = cos(m.*lam(idx));
        osin  = sin(m.*lam(idx));
        % ----------------------------
        mFlm  = TSQ.*realsqrt((l-m+1).*(l-m+2).*2);
        mPlm  = TF.*(plm(l+1,m-1,th));
        mcos  = cos((m-1).*lam(idx));
        % ----------------------------
        dDV_dClm(idx,1,:) =    TK .* bsxfun(@times,mFlm,bsxfun(@times,mPlm,mcos)) - TK .* bsxfun(@times,pFlm,bsxfun(@times,pPlm,pcos));
        dDV_dClm(idx,2,:) =   -TK .* bsxfun(@times,pFlm,bsxfun(@times,pPlm,psin));
        dDV_dClm(idx,3,:) =-2.*TK .* bsxfun(@times,oFlm,bsxfun(@times,oPlm,ocos));
        % ----------------------------
        dDV_dSlm(idx,1,:) =   -TK .* bsxfun(@times,pFlm,bsxfun(@times,pPlm,psin));
        dDV_dSlm(idx,2,:) =    TK .* bsxfun(@times,mFlm,bsxfun(@times,mPlm,mcos)) + TK .* bsxfun(@times,pFlm,bsxfun(@times,pPlm,pcos));
        dDV_dSlm(idx,3,:) =-2.*TK .* bsxfun(@times,oFlm,bsxfun(@times,oPlm,osin));

    else
        pFlm  = TSQ.*realsqrt((l+m+1).*(l+m+2));
        pPlm  = TF.*(plm(l+1,m+1,th));
        pcos  = cos((m+1).*lam(idx));
        psin  = sin((m+1).*lam(idx));
        % ----------------------------
        oFlm  = TSQ.*realsqrt((l-m+1).*(l+m+1));
        oPlm  = TF.*(plm(l+1,m,th));
        ocos  = cos(m.*lam(idx));
        osin  = sin(m.*lam(idx));
        % ----------------------------
        mFlm  = TSQ.*realsqrt((l-m+1).*(l-m+2));
        mPlm  = TF.*(plm(l+1,m-1,th));
        mcos  = cos((m-1).*lam(idx));
        msin  = sin((m-1).*lam(idx));
        % ----------------------------
        dDV_dClm(idx,1,:) =    TK .* bsxfun(@times,mFlm,bsxfun(@times,mPlm,mcos)) - TK .* bsxfun(@times,pFlm,bsxfun(@times,pPlm,pcos));
        dDV_dClm(idx,2,:) =   -TK .* bsxfun(@times,mFlm,bsxfun(@times,mPlm,msin)) - TK .* bsxfun(@times,pFlm,bsxfun(@times,pPlm,psin));
        dDV_dClm(idx,3,:) =-2.*TK .* bsxfun(@times,oFlm,bsxfun(@times,oPlm,ocos));
        % ----------------------------
        dDV_dSlm(idx,1,:) =    TK .* bsxfun(@times,mFlm,bsxfun(@times,mPlm,msin)) - TK .* bsxfun(@times,pFlm,bsxfun(@times,pPlm,psin));
        dDV_dSlm(idx,2,:) =    TK .* bsxfun(@times,mFlm,bsxfun(@times,mPlm,mcos)) + TK .* bsxfun(@times,pFlm,bsxfun(@times,pPlm,pcos));
        dDV_dSlm(idx,3,:) =-2.*TK .* bsxfun(@times,oFlm,bsxfun(@times,oPlm,osin));

    end
    
    % prepare output and rotate if desired
    if strcmpi(drtype,'lscs')
        cp = cos(phi(idx));
        sp = sin(phi(idx));
        cl = cos(lam(idx));
        sl = sin(lam(idx));
        for ridx2 = 1:size(dDV_dClm,3)
            Re2l = [cp.*cl,  cp.*sl,    sp,  -sp.*cl,  -sp.*sl,   cp,  -sl,  cl,  zeros(size(sp))];
            dDV_dClm(idx,:,ridx2) = multmatvek(Re2l,squeeze(dDV_dClm(idx,:,ridx2)));
            dDV_dSlm(idx,:,ridx2) = multmatvek(Re2l,squeeze(dDV_dSlm(idx,:,ridx2)));
        end
    elseif any(size(drtype)==9)
        for ridx2 = 1:size(dDV_dClm,3)
            dDV_dClm(idx,:,ridx2) = multmatvek(drtype(idx,:),squeeze(dDV_dClm(idx,:,ridx2)));
            dDV_dSlm(idx,:,ridx2) = multmatvek(drtype(idx,:),squeeze(dDV_dSlm(idx,:,ridx2)));
        end
    end

    % CleanUp
    %clear idx l th TF TSQ pClm oClm mClm pSlm oSlm mSlm mPQ oPQ pPQ mcos ocos pcos msin osin psin cp sp cl sl
end % end for I=1:parts







