function C=multmatvek(A,B,trnsflag)

% MULTMATVEK mulitplicates a matrix and a vector, where each line of them
% represents a rotation matrix at certain time point.
%
% The multiplication A*B will be performed. This function is especially
% designed to support the multiplication of 3x3 rotation matrices.
% Sometimes this must be done for a time series of rotation matrices.
% Instead of using a for-loop it is possible to use this function. Of
% course, since Matlab only supports calculations for 2D-matrices, the
% format of the input matrices must be altered.
%
% The input format for the matrices A is as follows:
%     A(:,1) = A11(t)   for all t
%     A(:,2) = A12(t)   for all t
%     A(:,3) = A13(t)   for all t
%     A(:,4) = A21(t)   for all t
%     A(:,5) = A22(t)   for all t
%     A(:,6) = A23(t)   for all t
%     A(:,7) = A31(t)   for all t
%     A(:,8) = A32(t)   for all t
%     A(:,9) = A33(t)   for all t
% Analog for B!
%     B(:,1) = B1(t)    for all t
%     B(:,2) = B2(t)    for all t
%     B(:,3) = B3(t)    for all t
%
% Call: C = multmatvek(A,B,trnsflag);
%
% Parameter:
%   A         [n,9]   matrix
%   B         [n,3]   vector
%   trnsflag  [bool]  flag for using the transpose of A
%
% Output:
%   C         [n,3]   multiplication of A and B
%
%----------------------------------------------------------------------------
% uses none
%----------------------------------------------------------------------------
% revision history
% 21.02.2013, MW: changed transpose flag and brushed help text
%----------------------------------------------------------------------------
% remarks
%
%----------------------------------------------------------------------------
% Matthias Weigelt                                                 29.06.04

%%

% Constants
narginchk(2,3);
if nargin < 3, trnsflag = false; end
if ~islogical(trnsflag), error('trnsflag must be logical.'); end

% input check
[m,n] = size(A);
if (m~=9 && n~=9)
    error('Wrong input format for matrix A');
elseif (m==9 && n~=9)
    A=A';
end

[m,n] = size(B);
if (m~=3 && n~=3)
    error('Wrong input format for matrix B');
elseif (m==3 && n~=3)
    B=B';
end


% sorting
if trnsflag,
    A1 = A(:,[1 4 7]);  % equal to A matrix 1. column
    A2 = A(:,[2 5 8]);  % equal to A matrix 2. column
    A3 = A(:,[3 6 9]);  % equal to A matrix 3. column
else
    A1 = A(:,1:3);      % equal to A matrix 1. row
    A2 = A(:,4:6);      % equal to A matrix 2. row
    A3 = A(:,7:9);      % equal to A matrix 3. row
end

% 1.row of C
C1 = sum(A1.*B,2);
C2 = sum(A2.*B,2);
C3 = sum(A3.*B,2);

% form new format
C = [C1 C2 C3];

