function [ y, dydt ] = odeint(deriv, y0, dydt, t0, h, i_call, int_meth, abm_order, deriv_params)
   
% ODEINT integrates one step with step size dt from t0 and initial values y0
% with a chosen integrator. This is more or less a wrapper switching the chosen
% integration method

% Inputs:       deriv = function providing derivative of states, all
%               parameters of the deriv function are saved in the struct
%               "deriv_params", making it easy to change or add parameters
%               to the function without passing it to all other
%               integration functions.
%               y0 = initial conditions at time t0
%               dydt = derivatives = deriv results of all states for the last
%               abm_order steps 
%               t0 = time from which to integrate 
%               h = step size of integration, there are no intermediate steps!
%               dt = step size of integration, there are no intermediate steps!
%               i_call = calls of this function. In first call i_call = 1. This
%               is needed for the initialization of the ABM integrator because
%               abm_order steps need to be done with a single step scheme like
%               Runge-Kutta
%               int_meth = desired integration method
%               abm_order = order of ABM scheme
%               deriv_params = parameters needed by derivs, eg 
%                              field,data_plm,GM,ae,lmaxf,lmaxcs,noc
% 
% Output:       y = integrated variables at time t0 + dt
%               dydt = derivatives = deriv results of all states at time t0 + dt
% 
% Main functions
% - odeint_rk4: integration with Runge Kutta Order 4
% - odeint_dp8: integration with Runge Kutta Order Dormand Prince scheme order 8
% - odeint_abm: integration with ABM integrator with constant, chosen order
% - odeint_dp8_abm_init: initialization of ABM with DP8 
%  
% Written by Florian Wöske, ZARM Uni Bremen, 2018-04


%%
    if int_meth == 1 % Runge-Kutta 4
        y = odeint_rk4(deriv, y0, t0, h,  deriv_params);
        dydt = 0;
    elseif int_meth == 2 % Runge-Kutta DP8 
        y = odeint_dp8(deriv, y0, t0, h,  deriv_params);
        dydt = 0;
    elseif int_meth == 3 % ABM
        % initialization of ABM with RK
        if i_call < abm_order 
            % init with RK4 
%             [y, dydt_i] = odeint_rk4_abm_init(deriv, y0, t0, h,  deriv_params);
            % init with DP8 
            [y, dydt_i] = odeint_dp8_abm_init(deriv, y0, t0, h,  deriv_params);
            dydt(abm_order-i_call+1,:) = dydt_i;
        else
        % ABM integration after first initialization steps
        [y, dydt] = odeint_abm(deriv, y0, dydt, t0, h, abm_order,  deriv_params);
        end
    else
        error('The chosen integration method int_meth does not exist');
    end

end
