function [ y ] = odeint_rk4(deriv, y0, t0, h, deriv_params)

% Integrates one step with step size h from t0 and initial values y0 with
% simple 4th order Runge-Kutta (RK) integrator.
%
% Inputs:       deriv = function providing derivative of states, all
%               parameters of the deriv function are saved in the struct
%               "deriv_params", making it easy to change or add parameters
%               to the function without passing it to all other
%               integration functions.
%               y0 = initial conditions at time t0
%               t0 = time from which to integrate
%               h = step size of integration, there are no intermediate steps!
%               deriv_params = parameters needed by derivs, eg
%                              field,data_plm,GM,ae,lmaxf,lmaxcs,noc
%
% Output:       y = integrated variables at time t0 + dt
%
% Main functions
% - deriv
%
% Written by Florian Wöske, ZARM Uni Bremen, 2018-04

%%

% evaluate k1
dy= deriv(t0, y0, deriv_params);
   k1 = h*dy;
   y  = y0 + k1/2.0 ;

% evaluate k2
dy= deriv(t0+h/2.0, y, deriv_params);
   k2 = h*dy ;
   y = y0 + k2/2.0   ;

% evaluate k3
dy= deriv(t0+h/2.0, y, deriv_params);
   k3 = h*dy ;
   y  = y0 + k3   ;

% evaluate k4 and the result
dy= deriv(t0+h, y, deriv_params) ;
   k4 = h*dy;
   y = y0 + k1/6.0+k2/3.0+k3/3.0+k4/6.0 ;

end

