function [p,dp,ddp] = plm(l,m,th)

% PLM Fully normalized associated Legendre functions for a selected order M
%
% HOW p          = plm(l,th)			- assumes M=0
%     p          = plm(l,m,th)
%     [p,dp]     = plm(l,m,th)
%     [p,dp,ddp] = plm(l,m,th)
%
% IN  l  - degree (vector). Integer, but not necessarily monotonic.
%          For l < m a vector of zeros will be returned.
%     m  - order (scalar). If absent, m=0 is assumed.
%     th - co-latitude [deg] (vector)
% OUT p  - Matrix with Legendre functions. The matrix has length(TH) rows
%          and length(L) columns, unless L or TH is scalar. Then the output
%          vector follows the shape of respectively L or TH.
%    dp  - Matrix with first derivative of Legendre functions. The matrix
%          has length(TH) rows and length(L) columns, unless L or TH is
%          scalar. Then the output vector follows the shape of respectively
%          L or TH.
%   ddp  - Matrix with second derivative of Legendre functions. The matrix
%          has length(TH) rows and length(L) columns, unless L or TH is
%          scalar. Then the output vector follows the shape of respectively
%          L or TH.
%
% IMPORTANT NOTE: The derivatives are towards the latitude, i.e.
%                 dP/d\phi is calculated. In order to get the derivative
%                 towards the co-latitude one has to multiply with minus.
%                   ->  dP:d\theta      =   -dP:d\phi
%                   ->  d^2P:d\theta^2  =  d^2P:d\phi^2
%
% See also LEGPOL, YLM, IPLM
%--------------------------------------------------------------------------
% Nico Sneeuw, IAPG, TU-Munich                                    08/08/94
%--------------------------------------------------------------------------
% Uses none
%--------------------------------------------------------------------------
% Revision history:
%  - NS09/06/97: help text brushed up
%  - NS13/07/98: Pmm non-recursive anymore
%  - NS0299:     further help text brush-up
%  - MW13/08/04: extension for first derivative
%  - MW24/11/04: speed up calculation
%  - MW04/04/08: extension for second derivative
%--------------------------------------------------------------------------

% Some input checking.
if nargin == 2
   th = m;
   m  = 0;
end
if min(size(l)) ~= 1,  error('Degree l must be vector (or scalar)'), end
if any(rem(l,1) ~= 0), error('Vector l contains non-integers.'), end
if max(size(m)) ~= 1,  error('Order m must be scalar.'), end
if rem(m,1)     ~= 0,  error('Order m must be integer.'), end

% Preliminaries.
lcol = size(l,2);
trow = size(th,1);
lmax = max(l);
if lmax < m,
    p = zeros(length(th),length(l));
    dp = zeros(length(th),length(l));
    ddp = zeros(length(th),length(l));
    return
end
n    = length(th);				% number of latitudes
t    = th(:)*pi/180;
x    = cos(t);
y    = sin(t);
lvec = l(:)';					% l can be used now as running index.

% Recursive computation of the temporary matrix ptmp, containing the Legendre
% functions in its columns, with progressing degree l. The last column of
% ptmp will contain zeros, which is useful for assignments when l < m.
ptmp  = zeros(n,lmax-m+2);
if nargout >= 2,                  % first derivative needs also P_{n,m+1} and P_{n,m-1}
    ptmp_m1 = zeros(n,lmax-m+3);
    ptmp_p1 = zeros(n,lmax-m+1);
    dptmp   = zeros(n,lmax-m+2);
end
if nargout == 3,                  % second derivative needs also dP_{n,m+1} and dP_{n,m-1}
    dptmp_m1 = zeros(n,lmax-m+3);
    dptmp_p1 = zeros(n,lmax-m+1);
    ptmp_m2  = zeros(n,lmax-m+4);  % but these first derivative need dP_{n,m+2} and dP_{n,m-2}
    ptmp_p2  = zeros(n,lmax-m);
    ddptmp   = zeros(n,lmax-m+2);
end

%--------------------------------------------------------------------
% sectorial recursion: PM (non-recursive, though)
%--------------------------------------------------------------------
ptmp(:,1) = secrecur(m,y);
if nargout >= 2,  % frist derivative needs preceding and subsequent element
    if m > 0,    ptmp_m1(:,1) = secrecur(m-1,y); end     % preceding elements
    if m < lmax, ptmp_p1(:,1) = secrecur(m+1,y); end     % subsequent elemtens
end
if nargout == 3,  % second derivative needs P_{n,m+2} and P_{n,m-2} as well
    if m > 1,      ptmp_m2(:,1) = secrecur(m-2,y); end   % preceding elements
    if m < lmax-1, ptmp_p2(:,1) = secrecur(m+2,y); end   % subsequent elemtens
end

%--------------------------------------------------------------------
% l-recursion: P
%--------------------------------------------------------------------
ptmp = lrecur(ptmp,x,m,lmax);
if nargout >= 2,  % frist derivative needs preceding and subsequent element
    if m > 0,    ptmp_m1 = lrecur(ptmp_m1,x,m-1,lmax); end    % preceding elements
    if m < lmax, ptmp_p1 = lrecur(ptmp_p1,x,m+1,lmax); end    % subsequent elemtens
end
if nargout == 3,  % second derivative needs P_{n,m+2} and P_{n,m-2} as well
    if m > 1,      ptmp_m2 = lrecur(ptmp_m2,x,m-2,lmax); end  % preceding elements
    if m < lmax-1, ptmp_p2 = lrecur(ptmp_p2,x,m+2,lmax); end  % subsequent elemtens
end

%--------------------------------------------------------------------
% now compute the derivatives
%--------------------------------------------------------------------
if nargout >= 2,                                   % first derivative
    dptmp = derivALF(dptmp,ptmp_m1,ptmp_p1,m,lmax);
end
if nargout == 3,                                   % second derivative
    if m > 0,    dptmp_m1 = derivALF(dptmp_m1,ptmp_m2,ptmp,m-1,lmax); end
    if m < lmax, dptmp_p1 = derivALF(dptmp_p1,ptmp,ptmp_p2,m+1,lmax); end
    ddptmp = derivALF(ddptmp,dptmp_m1,dptmp_p1,m,lmax);
end

%--------------------------------------------------------------------
% The Legendre functions have been computed. What remains to be done, is to
% extract the proper columns from ptmp, corresponding to the vector lvec.
% If l or theta is scalar the output matrix p reduces to a vector. It should
% have the shape of respectively theta or l in that case.
%--------------------------------------------------------------------
lind       = (lvec < m);			        % index into l < m
pcol       = lvec - m + 1;			        % index into columns of ptmp
pcol(lind) = (lmax-m+2)*ones(sum(lind),1);	% Now l < m points to last col.
p          = ptmp(:,pcol);			        % proper column extraction
if nargout >= 2,
    dp = -dptmp(:,pcol);                    % proper column extraction
end
if nargout == 3,
    ddp = ddptmp(:,pcol);                   % proper column extraction
end
if max(size(lvec))==1  && min(size(th))==1 && (trow == 1),
    p = p';
    if nargout >= 2, dp  = dp';  end
    if nargout == 3, ddp = ddp'; end
end
if max(size(th))==1 && min(size(lvec))==1  && (lcol == 1),
    p = p';
    if nargout >= 2, dp  = dp';  end
    if nargout == 3, ddp = ddp'; end
end

% *************************************************************************
%                            INLINE FUNCTIONS
% *************************************************************************
% function for the sectorial recursion, non-recursive though
function out = secrecur(m,y)
if m == 0
   fac = 1;
else
   mm  = 2*(1:m);
   fac = sqrt(2*prod((mm+1)./mm));
end
out = fac*y.^m;             % The 1st column of ptmp.
% -------------------------------------------------------------------------

% function for the l-recursion
function in = lrecur(in,x,m,lmax)
for l = m+1:lmax
   col   = l - m + 1;			% points to the next column of ptmp
   root1 = realsqrt( (2*l+1)*(2*l-1)/((l-m)*(l+m)) ) ;
   root2 = realsqrt( (2*l+1)*(l+m-1)*(l-m-1) / ( (2*l-3)*(l-m)*(l+m) ) );

   % recursion
   if l == m+1
       in(:,col) = root1 *x.*in(:,col-1);
   else
       in(:,col) = root1 *x.*in(:,col-1) - root2 *in(:,col-2);
   end
end
% -------------------------------------------------------------------------

% function to calculate the derivate
function in = derivALF(in,miin,plin,m,lmax)

l = (m:lmax+1);
if m == 0
    in(:,1) = 0;
    if lmax > m, in(:,2:end) = bsxfun(@times,plin,-realsqrt((l(2:end)+1).*l(2:end)./2)); end %   (-ones(n,1)*realsqrt((l(2:end)+1).*l(2:end)./2)).*plin; end
elseif m == 1
    in(:,1) = miin(:,2);
    if lmax > m, in(:,2:end) =  bsxfun(@times,miin(:,3:end),realsqrt((l(2:end)+1).*l(2:end)./2)) - ...
            0.5.*bsxfun(@times,plin,realsqrt((l(2:end)-1).*(l(2:end)+2))); end
elseif m == lmax
    in(:,1) = realsqrt(m./2).*miin(:,2);
else
    in(:,1) = realsqrt(m./2).*miin(:,2);
    if lmax > m, in(:,2:end) = 0.5.*bsxfun(@times,miin(:,3:end),realsqrt((l(2:end)+m).*(l(2:end)-m+1))) - ...
            0.5.*bsxfun(@times,plin,realsqrt((l(2:end)-m).*(l(2:end)+m+1))); end
end







