%% Single point Legendre polynomial
function [p,dp,ddp] = plmsp(lmax,th,data)
% PLMSP - Fully normalized associated Legendre functions for all degree and
% order but in a single points. The output is in sc-format.
%
% HOW [p1]          = plmsp(l,th)
%     [p1,dp1]      = plmsp(l,th)
%     [p1,dp1,ddp1] = plmsp(l,th)
%
% IN  lmax - maximum degree
%       th - co-latitude [rad] (1,1)
%     data - components for calculation from initplm
%
% OUT p  - Matrix with Legendre functions. The matrix is in the sc-format.
%    dp  - Matrix with first derivative of Legendre functions. The matrix
%          is in the sc-format.
%   ddp  - Matrix with second derivative of Legendre functions. The matrix
%          is in the sc-format.
%
% IMPORTANT NOTE: The derivatives are towards the latitude, i.e.
%                 dP/d\phi is calculated. In order to get the derivative
%                 towards the co-latitude one has to multiply with minus.
%                   ->  dP:d\theta      =   -dP:d\phi
%                   ->  d^2P:d\theta^2  =  d^2P:d\phi^2

%%

% Preliminaries
x = cos(th);
y = sin(th);

% sectorial recursion: PM (non-recursive, though)
ptmp1 = diag(data.fac.*(y(1).^(0:data.usem)));        % 1st column of ptmp1

% l-recursion: P
for idx = 1:data.usem
    if idx == 1
        data.didx    = data.didx(1:end-1);
        didx2        = data.didx+1;
        ptmp1(didx2) = data.root1(data.didx) .* x(1) .* ptmp1(data.didx);
    else
        didx0        = data.didx(1:end-1);
        data.didx    = didx2(1:end-1);
        didx2        = data.didx+1;
        ptmp1(didx2) = data.root1(data.didx) .* x(1) .* ptmp1(data.didx) -  data.root2(didx0) .* ptmp1(didx0);
    end
end

% First derivative
%--------------------------------------------------------------------
if nargout >= 2
    dp1      = zeros(size(ptmp1));
    dp1(:,1) = data.droot1(:,1).*ptmp1(:,2);                                      % Ordnung m = 0
    if lmax > 0
        dp1(:,2) = (data.droot1(:,2).*ptmp1(:,1) - data.droot2(:,2).*ptmp1(:,3))./2;     % Ordnung m = 1
        dp1(2,2) = ptmp1(2,1);                                               % P11
        if lmax > 1
            dp1(:,3:end-1) = (data.droot1(:,3:end-1).*ptmp1(:,2:end-2) - data.droot2(:,3:end-1).*ptmp1(:,4:end))./2;
        end
    end
end

% Second derivative
%--------------------------------------------------------------------
if nargout == 3
    ddp1      = zeros(size(ptmp1));
    ddp1(:,1) = data.droot1(:,1).*dp1(:,2);                                      % Ordnung m = 0
    if lmax > 0
        ddp1(:,2) = (data.droot1(:,2).*dp1(:,1) - data.droot2(:,2).*dp1(:,3))./2;       % Ordnung m = 1
        ddp1(2,2) = dp1(2,1);                                               % P11
        if lmax > 1
            ddp1(:,3:end-1) = (data.droot1(:,3:end-1).*dp1(:,2:end-2) - data.droot2(:,3:end-1).*dp1(:,4:end))./2;
        end
    end
end

% Create output
%--------------------------------------------------------------------
if nargout == 1,
    p = [fliplr(ptmp1(:,2:end)) ptmp1];
elseif nargout == 2,
    p  =[fliplr(ptmp1(1:end-1,2:end-1)) ptmp1(1:end-1,1:end-1) ];
    dp =[fliplr( -dp1(1:end-1,2:end-1))  -dp1(1:end-1,1:end-1) ];
elseif nargout == 3,
    p  =[fliplr(ptmp1(1:end-2,2:end-2)) ptmp1(1:end-2,1:end-2) ];
    dp =[fliplr( -dp1(1:end-2,2:end-2))  -dp1(1:end-2,1:end-2) ];
    ddp=[fliplr( ddp1(1:end-2,2:end-2))  ddp1(1:end-2,1:end-2) ];
end
end
