function out = vec2cs(nm,inC,inS)

% VEC2CS rearranges a vector shaped set of spherical harmonic
% coefficients into cs-format. The vector order is given by the ordering
% vector.
%
% CALL:  out = vec2cs(nm,inC,inS)
%
% PARAMETER:
%   nm    [2,k]   ordering vector
%         [1,1]   maximum degree - degree-wise ordering is assumed.
%   inC   [1,k]   coefficients in vector shape: cosine part
%   inS   [1,k]   coefficients in vector shape: sine part
%
% OUTPUT:
%   out   [n,m]   output in cs- or sc- format
%
% See also: cs2vec
%
% Uses functions:
% - sc2cs
%
% Written by Matthias Weigelt 2007-05-02
%--------------------------------------------------------------------------
% revision history
%  12.03.2013, MW: change input lmax to nm-vector
%--------------------------------------------------------------------------

%% Input check
narginchk(3,3);
if size(nm,1)~=2 && ~isscalar(nm), error('nm must be a 2 x k vector or a scalar.');    end
if ~isscalar(nm),
    if numel(inC)~=size(nm,2), error('Dimension of nm and inC do not match.');         end
    if numel(inS)~=size(nm,2), error('Dimension of nm and inS do not match.');         end
else
    lmax = nm;
    nm   = sum(1:lmax+1);  % number of elements
    if numel(inC)~=nm, error('Dimension of nm and inC do not match.');         end
    if numel(inS)~=nm, error('Dimension of nm and inS do not match.');         end
end

%% Preparation
% perpare the degree and order vector
if isscalar(nm),
    idx  = 1;
    nm   = zeros(2,nm);
    for l = 1:lmax+1
        for m = 1:l
            nm(1,idx) = l-1;
            nm(2,idx) = m-1;
            idx = idx+1;
        end
    end
end

%% Sort coefficients
lmax = max(nm(1,:));
outC = zeros(lmax+1);
outS = zeros(lmax+1);
for idx = 1:size(nm,2);
    outC(nm(1,idx)+1,nm(2,idx)+1) = inC(idx);
    outS(nm(1,idx)+1,nm(2,idx)+1) = inS(idx);
end
out = sc2cs([fliplr(outS(:,2:end)) outC]);

