%% GRACE gravity field recovery (GFR) using Energy Balance Approach (EBA)
% This is the main m-file preforming the gravity field estimation.
% All necessary parameters and data are specified in the following.
%
% Written by Neda Darbeheshti, AEI, 2019-06.

clear all
format longg;

%% variable inputs
global e12 en er

% add folder with all functions
Release2_Path = pwd;
addpath(genpath(Release2_Path));
EBA_Path = fullfile(Release2_Path,'/../function_eba/');
addpath(genpath(EBA_Path));
GFR_Path = fullfile(Release2_Path,'/../function_gfr/');
addpath(genpath(GFR_Path));

% folder where observation data are stored
% Use twelve days of data
Input_Observation_Path = fullfile(Release2_Path,'/../input_data','MockData_do20_nod12');
% Use four days of data (optional)
% Input_Observation_Path = fullfile(Release2_Path,'/../input_data','MockData_do10_nod4');
addpath(genpath(Input_Observation_Path));

% order of spherical harmonic coefficients to be estimated
n_max = 20;

% number of days
nod = 12;

% number of observations in each arc (one day, 17280 is the standard)
mKBR = 17280;

%% fixed inputs

% GM and Earth radius of gravity field
a = 6378136.3; %m
GM = 3.9860044150e+14;
we0=7.29211514670698*10^(-5); % rad/s

% size of gravity fields in vec format
length_N=(n_max/2+1)*(n_max+1)*2-(n_max+1);

% initialize for normal equations
N = zeros(length_N+1,length_N+1);
n0 = zeros(length_N+1,1);

% initialze one column for energy constant
A0 = ones(3600*24/5,1);

%% load observations data for all days from files

t0 = [2005 05 01 0 0 2];
t0i = t0;
tic
figure

for day=1:nod
    if day >1
        t0i(3)=t0i(3)+1;
    end
    date = time2str(t0i);

    % load GNV data GRACE A
    data_GNV = ['GNV1B_', date, '_A_02.asc'];
    data_file = fullfile(Input_Observation_Path, date, data_GNV);
    GNVA = readGNV(data_file);

    % load GNV data GRACE B
    data_GNV = ['GNV1B_', date, '_B_02.asc'];
    data_file = fullfile(Input_Observation_Path, date, data_GNV);
    GNVB = readGNV(data_file);

    % load KBR data
    data_KBR = ['KBR1B_', date, '_X_02.asc'];
    data_file = fullfile(Input_Observation_Path, date, data_KBR);
    KBRB = readKBR(data_file);

    % transformation to the Earth_fixed frame
    RotMat= Ri2e(GNVA(:,1));
    RotMat_dot= Ri2e_dot(GNVA(:,1));
    rA_e = threeDmulti_rot(RotMat,GNVA(:,2:4),'forward');
    vA_e = threeDmulti_rot(RotMat,GNVA(:,5:7),'forward')+threeDmulti_rot(RotMat_dot,GNVA(:,2:4),'forward');
    rB_e = threeDmulti_rot(RotMat,GNVB(:,2:4),'forward');
    vB_e = threeDmulti_rot(RotMat,GNVB(:,5:7),'forward')+threeDmulti_rot(RotMat_dot,GNVB(:,2:4),'forward');

    % check if the transformation is correct
    plot3(vA_e(:,1),vA_e(:,2),vA_e(:,3))
    hold on

% state vector
sa = [rA_e vA_e];
sb = [rB_e vB_e];

% calculate potential observable
v12sst0g = potential_EBAs(KBRB(:,3),sa,sb,we0);

[lon1, lat1, h1] = cart2sph(sa(:,1),sa(:,2),sa(:,3));
[lon2, lat2, h2] = cart2sph(sb(:,1),sb(:,2),sb(:,3));

%% Computation of design matrix A
A2g = designmatrixn(lon2,lat2,h2,n_max,GM,a);
A1g = designmatrixn(lon1,lat1,h1,n_max,GM,a);

A = A2g-A1g;
A = [A, A0];

% Update N:normal equation matrix
N = N + A'*A;

% Update n0:normal equation vector
n0 = n0+ A'*v12sst0g';

fprintf('done with day %d \n', day)
end

%% Cholesky decomposition
L = chol(N, 'lower');

z0 = L\n0;
x0 = L'\z0;

%% estimate residuals
yhat = A*x0;
eps = v12sst0g'-yhat;

toc

%% writing degree variances
[cnm, snm] = sortCoefficients(x0,n_max);

kk = 0;

for i=1:length(cnm)
    for j=1:length(snm)
        if j<=i
           kk=kk+1;
           xegm(kk,1)=i-1;
           xegm(kk,2)=j-1;
           xegm(kk,3)=cnm(i,j);
           xegm(kk,4)=snm(i,j);
        end
    end
end

%% save results in a file
FileName=[Release2_Path,'/results/EBA','_do_', num2str(n_max), '_nods_', num2str(nod),'_',datestr(now, 'dd-mmm-yyyy'),'.mat'];
save(FileName,'xegm','eps','GM','a','n_max','e12', 'en', 'er','rA_e');
